# -*- coding: utf-8 -*-
#
# Copyright (C) 2020-2021 CERN.
# Copyright (C) 2021 Northwestern University.
#
# Invenio-Vocabularies is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Version information for Invenio-Vocabularies.

This file is imported by ``invenio_vocabularies.__init__``,
and parsed by ``setup.py``.
"""

__version__ = "0.7.4"
